## -*-Tcl-*-
 # ###################################################################
 #  AlphaTcl - extension to interact with sourceforge
 # 
 #  FILE: "sourceforgeMenu.tcl"
 #                                    created: 09/29/2001 {10:58:06 AM} 
 #                                last update: 04/24/2002 {11:02:54 AM} 
 #  Author: Vince Darley
 #  E-mail: vince@santafe.edu
 #          317 Paseo de Peralta, Santa Fe
 #     www: http://www.santafe.edu/~vince/
 #  
 # ###################################################################
 ##

# menu declaration
alpha::menu sourceforgeMenu 0.3.1 global SF {
    
} {
    sourceforgeMenu
} {
    
} maintainer {
    {Vince Darley} vince@santafe.edu http://www.santafe.edu/~vince/
} help {
    Provides functionality to aid interactions with the open
    source repository sourceforge.
} uninstall {this-file}

proc sourceforgeMenu {} {}

namespace eval sf {}

proc sf::buildMenu {args} {
    global sourceforgeMenu sfmodeVars sfprojects
    set items [list "goToSourceForge"]
    if {[array size sfprojects] > 0} {
	lappend items \
	  [menu::makeFlagMenu project list project sfmodeVars]
    }
    lappend items "editProjects" "(-)"
    if {[string length $sfmodeVars(project)]} {
	lappend items "$sfmodeVars(project)&"
    }
    lappend items "browseCvs" "viewCvsChangesToFile" "viewBugNumber" 
    
    Menu -n $sourceforgeMenu -p sf::MenuProc $items
}

# If the array exists, then these should've been set at some point
# in the past, so we don't do it again, in case the user actually
# wanted to delete them!
if {![array exists sfprojects]} {
    ensureset sfprojects(Tcl) "sfProjectName tcl sfGroupId 10894"
    ensureset sfprojects(AlphaTcl) "sfProjectName alphatcl sfGroupId 16416"
    # save them.
    prefs::modified sfprojects(Tcl) sfprojects(AlphaTcl)
}

# The current project
newPref var project Tcl sf sf::buildMenu sfprojects array

sf::buildMenu

proc sf::MenuProc {menu item} {
    global sfprojects
    if {[info exists sfprojects($item)]} {
	sf::goToProject $item
    } else {
	menu::generalProc sf $item
    }
}

proc sf::editProjects {} {
    global sfprojects sfmodeVars
    set res [dialog::editGroup -array sfprojects -delete ask \
      -new sf::newProjectDetails \
      -title "Edit sourceforge project information" \
      -current $sfmodeVars(project) \
      [list "sfProjectName" variable "Sourceforge project name"\
      "This is the name by which sourceforge refers to the project"] \
      [list "sfGroupId" variable "Sourceforge group id"\
      "This is the number (usually 5 digit) which sourceforge uses\
      to access further information on this project, usually using\
      http lookups with '?group_id=xxxxx'"] \
      [list "sfLocalDir" folder "Head of local copy of project"\
      "Location of checked-out version of the project"]]
    # This works whether projects have been modified or deleted.
    foreach proj $res {
	prefs::modified sfprojects($proj)
    }
    if {[llength $res]} {
	# Should now rebuild the menu in case we've added or
	# removed a project.
	sf::buildMenu
    }
}

proc sf::newProjectDetails {} {
    set name [getline "Descriptive name of new project"]
    if {[string length $name]} {
	return [list $name [list sfProjectName "" sfGroupId ""]]
    } else {
	return ""
    }
}

proc sf::goToSourceForge {} {
    url::execute "http://sourceforge.net"
}

proc sf::project {{proj ""}} {
    if {![string length $proj]} { 
	global sfmodeVars
	return $sfmodeVars(project)
    }
    return $proj
}

proc sf::projectInfo {{proj ""}} {
    global sfmodeVars sfprojects
    if {![string length $proj]} { 
	set proj $sfmodeVars(project)
    }
    return $sfprojects($proj)
}

proc sf::ChooseProjectFor {{filename ""}} {
    if {$filename == ""} {
	set filename [win::Current]
    }
    if {$filename == ""} { 
	status::msg "No window open!"
	return 
    }
    global sfprojects
    foreach proj [array names sfprojects] {
	set projectInfo [sf::projectInfo $proj]
	set dir [lindex $projectInfo 5]
	if {$dir == "" || ![file exists $dir]} {
	    continue
	}
	if {[file::pathStartsWith $filename $dir]} {
	    return $proj
	}
    }
    return ""
}

set sf::cvsCgiRoot "http://cvs.sourceforge.net/cgi-bin/viewcvs.cgi/"

proc sf::goToProject {{proj ""}} {
    set projectInfo [sf::projectInfo $proj]
    url::execute "http://sourceforge.net/projects/[lindex $projectInfo 1]"
}

proc sf::browseCvs {{proj ""}} {
    global sf::cvsCgiRoot
    set projectInfo [sf::projectInfo $proj]
    url::execute "[set sf::cvsCgiRoot][lindex $projectInfo 1]/"
}

proc sf::viewCvsChangesToFile {} {
    set filename [win::Current]
    if {$filename == ""} { 
	status::msg "No window open!"
	return 
    }
    set proj [sf::ChooseProjectFor $filename]
    if {![string length $proj]} {
	status::msg "File not part of any known project"
	return
    }
    set projectInfo [sf::projectInfo $proj]
    set local [lindex $projectInfo 5]
    set dir [file dirname $filename]
    # It is more robust to look for the cvs repository, because that will cope
    # with the situation where the file we're looking at is part of a module inside
    # the repository, and not the whole checkout.
    if {[file exists [file join $dir CVS Repository]]} {
	set repository [string trim [file::readAll [file join $dir CVS Repository]]]
	set repository [file join $repository [file tail $filename]]
    } else {
	if {![file::pathStartsWith $filename $local repository]} {
	    status::msg "File not part of $proj"
	    return
	}
    }
    set rest [string trimleft [file::toUrl $repository] file:///]
    
    global sf::cvsCgiRoot
    set url "[set sf::cvsCgiRoot][lindex $projectInfo 1]/$rest"
    status::msg "Going to $url"
    url::execute $url
}

proc sf::viewBugNumber {{proj ""} {bug ""}} {
    if {$bug == ""} { set bug [getline "View [sf::project $proj] project bug#"] }
    if {$bug == ""} { return }
    set group [lindex [sf::projectInfo $proj] 3]
    set http "http://sourceforge.net/tracker/index.php"
    append http "?func=detail&aid=${bug}&group_id=${group}&atid=1${group}"
    url::execute $http
}
