# -*-Tcl-*- (install) (nowrap)
# 
# File : ASmode.tcl
# 	        Created : 2002-03-10 11:54:39
#     Last modification : 2002-03-26 22:36:37
# Author : Bernard Desgraupes
# e-mail : <bdesgraupes@easyconnect.fr>
# Web-page : <http://webperso.easyconnect.fr/bdesgraupes/alpha.html>
# Description :
#     AppleScript mode is useful for editing, compiling, decompiling,  executing,
#     processing scripts written in the AppleScript language. 
#     This is a complete rewriting of  the  previous  AppleScript  mode  by  John
#     Sarapata <sarapata_john@jpmorgan.com>.
#     Please read the doc in the "AppleScript Help" file (it is  located  in  the
#     Help menu once the package is installed) and the AppleScript mode tutorial.
#  
# (c) Copyright : Bernard Desgraupes, 2002
#     All rights reserved. This software is free software.  See  licensing  terms
#     in the AppleScript Help file.
#     
#     This file is part of the AppleScript mode package.


alpha::mode Scrp 2.0.2 appleScriptMenu {*.script *.scr *.scpt *.ascr} {
    appleScriptMenu electricTab
} {
    alpha::package require AlphaTcl 7.4
    addMenu appleScriptMenu "331" Scrp
} maintainer {
    "Bernard Desgraupes" <bdesgraupes@easyconnect.fr>  <http://webperso.easyconnect.fr/bdesgraupes/alpha.html> 
} uninstall this-file  help {file "AppleScript Help"} requirements {
    if {[info tclversion] < 8.0} {
	error "AppleScript Menu requires Tcl 8"
    }
} 

# Only register these if we've actually loaded the mode or menu at least once.
set dimmitemslist [list  "compile" "execute" "lineContinuation" "addComment" ]
foreach item $dimmitemslist {
    hook::register requireOpenWindowsHook [list "331" $item] 1	
}
unset dimmitemslist

namespace eval Scrp {}

# Check for Script folder. Create it if missing.
if {![file exists [file join $HOME AppleScripts]]} {
    file mkdir [file join $HOME AppleScripts]
} 

# Set up the mode specific preferences
newPref	v wordWrap {0} Scrp
newPref	f autoMark {0} Scrp
newPref	v prefixString {-- } Scrp
newPref	v leftFillColumn {3} Scrp
newPref	v funcExpr {^on[ \t]+([^ \t]+)\(} Scrp
newPref	v parseExpr {^[^ \t]+[ \t]+(\w+)} Scrp
newPref	v wordBreak {\w+} Scrp
newPref	v wordBreakPreface {\W} Scrp
# Set this flag if you want AppleScript mode to take care
# of descriptive comments stored as a TEXT resource in the
# compiled scripts.
newPref	f includeComments {1} Scrp
# Set this flag if you want AppleScript mode to take care
# of dynamic scripting terminology (i-e when an 'scsz' 
# resource is present) when trying to open a dictionary.
# This is useful mostly for applications with scriptable plug-ins.
newPref	f launchToGetTerminology {0} Scrp
# Default colo(u)rs
newPref v commentColor red Scrp
newPref v keywordColor blue Scrp
newPref v stringColor green Scrp


set Scrp::escapeChar "\\"
set Scrp::quotedstringChar "\""
set Scrp::lineContinuationChar ""

set Scrp::commentCharacters(General) "--"
# set Scrp::commentCharacters(Paragraph) [list "(* " " *)" " * "]
set Scrp::commentCharacters(Paragraph) [list "-- " "  --" " -- "]
set Scrp::commentCharacters(Box) [list "(*" 2 "*)" 2 "*" 3]

# Dummy proc
proc appleScriptMenu {} {}


# Menu building procs
# -------------------

proc Scrp::buildScriptsList {} {
    global HOME
    set scriptsList [glob -nocomplain -type osas -dir [file join $HOME AppleScripts] *]
    set appsList [glob -nocomplain -type APPL -dir [file join $HOME AppleScripts] *]
    set ma [list "Rebuild scripts list" "(-"]
    foreach f [lsort -ignore [concat $scriptsList $appsList]] {
	lappend ma "<E<S[file tail $f]"
	if {[expr [info tclversion] >= 8.0]} {
	    lappend ma "<S<Iedit [file tail $f]"
	}
    } 
    return $ma
}


proc menu::buildScrpMenu {} {
    if {[expr [info tclversion] >= 8.0]} {
	eval Menu -n "331" -p Scrp::menuProc \{ "newScript" "openScript" "compile" "execute"\
	  "run" "(-" "openDictionary" "(-" "addComment" "lineContinuation" "appleScriptTutorial" "(-"\
	  \{Menu -m -n "/\x1escripts^|" -p Scrp::scriptsFolderProc \{[Scrp::buildScriptsList]\}\}\}
    } else {
	eval Menu -n "331" -p Scrp::menuProc \{ "newScript" "run" "(-" \
	   "addComment" "lineContinuation" "appleScriptTutorial" "(-"\
	  \{Menu -m -n scriptsFolder -p Scrp::scriptsFolderProc \{[Scrp::buildScriptsList]\}\}\}
    }
}

# # (Alpha7 note) With AS Editor's icon as resource 'ICON' ID 264:
# 	  \{Menu -m -n "/\x1escriptsFolder^8" -p Scrp::scriptsFolderProc \{[Scrp::buildScriptsList]\}\}\}


# Build the menu
menu::buildScrpMenu


proc Scrp::menuProc {menu item} {
    eval Scrp::$item
}


proc Scrp::scriptsFolderProc {menu item} {
    global HOME
    if {$item == "Rebuild scripts list"} {
	menu::buildScrpMenu
    } else {
	if {[regexp "edit (.*)" $item dumm item]} {
	    Scrp::decompile [file join $HOME AppleScripts $item]
	} else {
	    Scrp::runScript $item
	}
    }
}


# Menu items procs
# ----------------
proc Scrp::newScript {} {
    global prefixString ScrpmodeVars
    catch {prompt "Name of the new script" newscript.scpt} scrname
    if {$scrname == "cancel"} {return} 
    new -n $scrname
    newMode Scrp
    if {$ScrpmodeVars(includeComments)} {Scrp::addComment} 
}


proc Scrp::appleScriptTutorial {} {
        help::openExample "AppleScript Example" 
}


# Syntax coloring
# ---------------
set ScrpKeyWords {
    after and as back before begin begins by close considering contains copy
    count data delete div does duplicate eighth else end ends equal equals
    error every exists fifth first fourth from front get global greater if
    ignoring in is it last launch less local make me middle mod move ninth
    not of on open or pi print prop property quit reference repeat result return
    run save script second set seventh sixth size some space start starts tab
    tell tenth than then third through thru timeout times to transaction try
    until while with 
}

regModeKeywords -e {--} -b {(*} {*)} -c $ScrpmodeVars(commentColor) \
  -k $ScrpmodeVars(keywordColor)  -s $ScrpmodeVars(stringColor)  Scrp $ScrpKeyWords



# Words completion
# ----------------

set completions(Scrp) {completion::cmd completion::electric}

set Scrpcmds $ScrpKeyWords

# We don't need the keywords anymore
unset ScrpKeyWords

# Abbreviations
set Scrpelectrics(tell)		" application \"\"\n\t\nend tell"
set Scrpelectrics(on)		" ()\n\t\nend "
set Scrpelectrics(if)		"  then\n\tend if"
set Scrpelectrics(try)		"\n\t\non error\n\t\n\treturn\nend try"
set Scrpelectrics(repeat)	" with  in \n\t\nend repeat"


# File Marking
# ------------
proc Scrp::MarkFile {} {
    global ScrpmodeVars
    set pos [minPos]
    while {![catch {search -s -f 1 -r 1 -m 0 -i 1 $ScrpmodeVars(funcExpr) $pos} res]} {
	set start [lindex $res 0]
	set end [lindex $res 1]
	set pos $end
	set text [getText $start $end]
	# Eliminate "on error" statements: they are not functions.
	regexp $ScrpmodeVars(funcExpr) $text dumm fnct
	if {$fnct!="error"} {
	    set inds($fnct) $res
	} 
    }
    
    if {[info exists inds]} {
	foreach f [lsort [array names inds]] {
	    setNamedMark $f [lineStart [pos::math [lineStart [lindex $inds($f) 0]] - 1]] \
	      [lindex $inds($f) 0] [pos::math [lindex $inds($f) 1] - 1]
	}
    }
}


proc Scrp::correctIndentation {args} {
    uplevel 1 ::correctBracesIndentation $args
}


# Option-clicking the title bar
# -----------------------------
proc Scrp::OptionTitlebar {} {
    global HOME
    set scriptsList [glob -nocomplain -type osas -dir [file join $HOME AppleScripts] *]
    set appsList [glob -nocomplain -type APPL -dir [file join $HOME AppleScripts] *]
    set scrlist ""
    foreach f [lsort -ignore [concat $scriptsList $appsList]] {
	lappend scrlist "[file tail $f]"
    } 
    return $scrlist
}


proc Scrp::OptionTitlebarSelect {item} {
    global HOME
    Scrp::decompile [file join $HOME AppleScripts $item]
}


# Key bindings
# ============
# We define AppleScript mode specific key bindings : all of them use 'ctrl-a'
# followed by a letter. For instance, to decompile and  edit  a  script,  hit
# 'ctrl-a' and then the letter 'o'.
Bind 'a' <z> prefixChar Scrp
# 'ctrl-a n'	to create a new script
Bind 'n' <A> {Scrp::newScript} Scrp
# 'ctrl-a o'	to decompile and open a script
Bind 'o' <A> {Scrp::openScript} Scrp
# 'ctrl-a c'	to compile the current window or selection
Bind 'c' <A> {Scrp::compile} Scrp
# 'ctrl-a e'	to execute (compile and run) the current window
Bind 'e' <A> {Scrp::execute} Scrp
# 'ctrl-a r'	to select and run a script
Bind 'r' <A> {Scrp::run} Scrp
# 'ctrl-a d'	to get the scripting terminology
Bind 'd' <A> {Scrp::openDictionary} Scrp
# 'ctrl-a x'	to clear the scripting contexts
Bind 'x' <A> {Scrp::clearContexts} Scrp
# 'ctrl-a l'	to insert the line continuation symbol ""
Bind 'l' <A> {Scrp::lineContinuation} Scrp
