# Automatically created by mode assistant, with input from Vince.
#
# Mode: Wiki


# Mode declaration.
#  first two items are: name version
#  then 'source' means source this file when we first use the mode
#  then we list the extensions, and any features active by default
alpha::mode Wiki 0.3.0 source {} WikiMenu {
    # Script to execute at Alpha startup
    addMenu WikiMenu Wiki
    if {[info tclversion] >= 8.0} {
	namespace eval WWW {}
	# Capture urls which match this regexp.  This pseudo-interface
	# is liable to be changed by WWW menu when we sort these things
	# out.
	set WWW::UrlActionhttp(.*/\[-a-zA-Z\]*wikit/(.*/)?\[0-9\]+@\$) Wiki::editUrl
	set WWW::UrlActionhttp(.*/tcl/\[0-9\]+@\$) Wiki::editUrl
	set WWW::UrlActionhttp(.*/wiki.tcl.tk/\[0-9\]+@\$) Wiki::editUrl
    }
}

# For Tcl 8
namespace eval Wiki {}

# This proc is called every time we turn the menu on.
# Its main effect is to ensure this code, including the
# menu definition below, has been loaded.
proc WikiMenu {} {}

# Mode preferences settings, which can be edited by the user (with F12)

newPref flag wordWrap 1 Wiki
newPref var fillColumn   70  Wiki
newPref var prefixString " " Wiki

# These are used by the ::parseFuncs procedure when the user clicks on
# the {} button in a file edited using this mode.  If you need more sophisticated
# function marking, you need to add a Wiki::parseFuncs proc

newPref var funcExpr {^'''[^\r\n]+'''} Wiki
newPref var parseExpr '''(.*)''' Wiki

# Declare a new item for 'New Document'
set {newDocTypes(New Wiki Project)} Wiki::newProject

# Colour the keywords, comments etc.
regModeKeywords Wiki {}
regModeKeywords -a -i "'" -I red Wiki {}
regModeKeywords -a -i "*" -I green Wiki {}
regModeKeywords -a -i "\]" -i "\[" -i "'" -I blue Wiki {}

hook::register savePostHook Wiki::justSavedHook Wiki
hook::register closeHook Wiki::closeHook Wiki

#  Wiki Bindings  #

# We need some of these because Alphatk doesn't always recognize non
# alphabetical characters in menu key bindings, i.e <control>-<shift> '*'
# doesn't seem to be a valid option.  Two versions given because Alpha 7/8
# only likes the first ones, while Alphatk requires the second.
Bind '1'   <zs>  {Wiki::menuProc wikiParagraph numberedItem}   Wiki
Bind '!'   <z>   {Wiki::menuProc wikiParagraph numberedItem}   Wiki
Bind '8'   <zs>  {Wiki::menuProc wikiParagraph bulletListItem} Wiki
Bind '*'   <z>   {Wiki::menuProc wikiParagraph bulletListItem} Wiki
Bind '-'   <zs>  {Wiki::menuProc wikiParagraph horizontalLine} Wiki
Bind '_'   <z>   {Wiki::menuProc wikiParagraph horizontalLine} Wiki
Bind ':'   <zs>  {Wiki::menuProc wikiParagraph definitionItem} Wiki
Bind '\;'  <z>   {Wiki::menuProc wikiParagraph definitionItem} Wiki

# <command>-<control>-<shift> [0-9] don't work in Alpha7/8, actually.  I'll
# file a bug report on this.  In the meantime, we'll have two different
# sets of bindings and use the option key.

if {[set alpha::macos]} {
    Bind '1'   <so>  {Wiki::menuProc wikiLine  numberedItem}   Wiki
    Bind '8'   <so>  {Wiki::menuProc wikiLine  bulletListItem} Wiki
    Bind '-'   <so>  {Wiki::menuProc wikiLine  horizontalLine} Wiki
} else {
    Bind '!'   <cz>  {Wiki::menuProc wikiLine  numberedItem}   Wiki
    Bind '*'   <cz>  {Wiki::menuProc wikiLine  bulletListItem} Wiki
    Bind '_'   <cz>  {Wiki::menuProc wikiLine  horizontalLine} Wiki
}

Bind '\''  <zs>  {Wiki::menuProc wikiText      unquote}        Wiki
Bind '\"'  <z>   {Wiki::menuProc wikiText      unquote}        Wiki

# We re-direct the 'Fill Paragraph' binding to our special proc.
Bind 'i'   <c>   {Wiki::paragraphFill} Wiki

# Handy navigation bindings.
Bind    up <sz>  {function::prev 0 0}  Wiki
Bind  left <sz>  {function::prev 0 1}  Wiki
Bind  down <sz>  {function::next 0 0}  Wiki
Bind right <sz>  {function::next 0 1}  Wiki


proc Wiki::editUrl {to args} {
    # To edit a Wiki page, you must add '@'
    if {[string index $to end] != "@"} {
	#append to "@"
    }
    set contents [url::contents $to]
    regexp "<title>(\[^\r\n\]*)</title>" $contents -> title
    regexp "<form (\[^>\]*)>" $contents -> form
    regexp "<textarea\[^>\]*>(.*)</textarea>" $contents -> textarea
    # Need to remove all html formatting (e.g. &quot;) from 'textarea'
    set textarea [quote::Unhtml $textarea]
    
    set name [temp::path wikitmp $title]
    file::writeAll $name $textarea 1
    
    set name [edit -w $name]
    newMode Wiki
    set name [win::Current]
    foreach attr $form {
	foreach {arg val} [split $attr =] {}
	set ::Wiki::${to}($arg) [string trim $val \"]
    }
    set ::Wiki::page($name) $to
}

proc Wiki::closeHook {name} {
    catch {
	set to $::Wiki::page($name)
	unset ::Wiki::page($name)
	unset ::Wiki::$to
    }
}

proc Wiki::justSavedHook {name} {
    if {![info exists ::Wiki::page($name)]} { return }
    # Gather information
    set to $::Wiki::page($name)
    upvar \#0 ::Wiki::${to} attr
    # What is the correct, general way to generate the correct
    # url to post to?
    if {[info exists attr(action)]} {
	regexp {^http://[^/]*} $to postTo
	append postTo $attr(action)
    } else {
	set postTo "[string trimright $to @].html"
    }
    # Post the new page back to the Wiki.  Tcl makes this
    # incredibly simple!  Thanks to the Tcler's Wiki for
    # appropriate code snippets to help.
    package require http
    status::msg "Posting the new version to $postTo ..."
    set query [::http::formatQuery C [getText [minPos] [maxPos]]]
    set http  [::http::geturl $postTo -query $query]
    status::msg "Posting the new version...done"
    # This will actually return the contents of the new web page
    # i.e. some html source code.
    set html  [::http::data $http]
    # cleanup
    ::http::cleanup $http

    status::msg "Edited page has been submitted successfully."
    
    # Now do something with the results.  If we were created by
    # the WWW mode, we might want to auto-reload whatever page
    # it was using the information we've got here (saves having
    # to reload it from the internet).
    WWW::webPageHasChanged "[string trim $to @].html" $html
}

# To write indentation code for your new mode (so your mode
# automatically takes advantage of the automatic indentation
# possibilities of 'tab', 'return' and 'paste'), you can take
# advantage of the shared proc ::indentLine.  All you need to write
# is a Wiki::correctIndentation proc, and as a
# starting point you can copy the code of the generic
# ::correctIndentation, found in indentation.tcl.

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::urlsAllowed"  ---
 # 
 # Used to determine if urls can be fetched.  The test here really should be
 # 
 #   expr {[info tclversion] < 8.0 ? 0 : 1}
 # 
 # but as of this writing 'httpCopy' will do a nasty freeze in Alpha8.0b1,
 # so we only offer this for Alphatk and AlphaX.  This should be changed
 # once Alpha8 issues are fixed.
 # 
 # --------------------------------------------------------------------------
 ## 

proc Wiki::urlsAllowed {} {
    global alpha::platform alpha::macos
    if {[set alpha::platform] == "alpha"} {
	if {[set alpha::macos] == 2} {
	    return 1
	}
	return 0
    }
    return 1
}

#  Wiki Projects  #

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::defaultTags"  ---
 # 
 # Define some default values for mark-up tags.  If new items are added (or
 # if any old ones are deleted), make sure that 'Wiki::editProjects' is
 # adjusted accordingly.  No need to include 'unverbatim' or 'unquote' here. 
 # Use real carriage returns rather than '\r' to put special glyph in
 # dialogs.
 # 
 # This proc has to be defined before any of the default wiki projects are
 # added below.
 # 
 # --------------------------------------------------------------------------
 ## 

proc Wiki::defaultTags {} {
    
    global Wiki::Projects
    
    if {[info exists Wiki::Projects(default)]} {
	set Wiki::Projects(default)
    } else {
	set Wiki::Projects(default) [list \
	  projectHome "" formatting "" \
	  horizontalLine [list {----
}] \
	  bulletListItem [list {   * }    ] \
	  numberedItem   [list {   1. }   ] \
	  definitionItem [list "\t" " :\t"] \
	  verbatim       [list { }        ] \
	  bold           [list {'''} {'''}] \
	  italics        [list {''}  {''} ] ]
	prefs::modified Wiki::Projects(default)
    }
}

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::projects"  ---
 # 
 # With no arguments, return a sorted list of all registered projects,
 # otherwise ensure that the project exists.  Returns '1' if the project
 # was created, otherwise '0'.
 # 
 # For each wiki project, we include most of the formatting mark-up tags here
 # because some wikis may vary.  Probably should have some auto-update
 # function to synchronize the current wiki page with its project, though
 # it's easy enough for the user to select the project manually.
 # 
 # We set the 'Wiki::ProjectsList' here, which is used as the list for
 # the WikimodeVars(project) list of options.
 # 
 # --------------------------------------------------------------------------
 ## 

proc Wiki::projects {args} {
    
    global Wiki::Projects Wiki::ProjectsList
    
    set projects [lsort -ignore [array names Wiki::Projects]]
    if {[llength $projects] > 1} {
	set Wiki::ProjectsList [concat [list "default" "-" ] \
	  [lremove [lsort -ignore [array names Wiki::Projects]] "default"]]
    } else {
	set Wiki::ProjectsList [list "default"]
    }
    if {![llength $args]} {
	set projects
    } elseif {[llength $args] != "2"} {
	error "Incorrect arguments: <name> <list of settings>"
    } elseif {[info exists Wiki::projects([lindex $args 0])]} {
	return 0
    } else {
	set name     [lindex $args 0]
	set settings [lindex $args 1]
	set Wiki::Projects($name) $settings
	prefs::modified Wiki::Projects($name)
	return 1
    }
}

# If the array exists, then these should've been set at some point in the
# past, so we don't do it again, in case the user actually wanted to delete
# them!  This will automatically set the "default" project, which will never
# be removed.  These should perhaps be moved into the alphadev menu and tcl
# mode, wrapped in a catch so that they won't throw an error if Wiki mode is
# not installed.  Don't use 'Wiki::defaultTags' here for these projects
# because our wikis have slightly different markup tags.  Don't use '\r'
# so that we can get the special glyph in the edit dialog.
if {![llength [Wiki::projects]]} {
    
    Wiki::defaultTags

    Wiki::projects "AlphaTcl" [list \
      projectHome "http://alphatcl.sourceforge.net/wikit/" \
      formatting  "http://alphatcl.sourceforge.net/wikit/6.html" \
      horizontalLine [list {----
}] \
      bulletListItem [list {   * }      ] \
      numberedItem   [list {   1. }     ] \
      definitionItem [list {   } { :   }] \
      verbatim       [list { }          ] \
      bold           [list {'''} {'''}  ] \
      italics        [list {''}  {''}   ] ]

    Wiki::projects "Tcl" [list \
      projectHome "http://mini.net/tcl/" \
      formatting  "http://mini.net/tcl/14.html" \
      horizontalLine [list {----
}] \
      bulletListItem [list {   * }      ] \
      numberedItem   [list {   1. }     ] \
      definitionItem [list {   } { :   }] \
      verbatim       [list { }          ] \
      bold           [list {'''} {'''}  ] \
      italics        [list {''}  {''}   ] ]
}

# The current Wiki project
newPref var project [lindex [Wiki::projects] 0] Wiki \
  Wiki::rebuildMenu Wiki::ProjectsList varitem

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::currentProject"  ---
 # 
 # Optionally sets a new project to be the current one, ensures that the
 # current project actually exists, and make some variables available to the
 # calling proc.
 # 
 # --------------------------------------------------------------------------
 ## 

proc Wiki::currentProject {{newProject ""}} {

    global WikimodeVars 

    upvar project  project
    upvar projects projects
    upvar settings settings

    # Create a list of all projects available.  If we don't have any projects
    # available, we still need to define the variables.
    if {![llength [set projects [Wiki::projects]]]} {
	set project ""
	set settings [concat \
	  [list projectHome "" formatting ""] [Wiki::defaultTags]]
	return
    } 
    # Determine the current project.
    if {[string length $newProject]} {
	# Make 'newProject' the current project.
	set WikimodeVars(project) $newProject
    }
    # Make sure that the current project exists.
    if {![lcontains projects [set project $WikimodeVars(project)]]} {
	set project [set WikimodeVars(project) [lindex $projects 0]]
    } 
    set settings [set Wiki::Projects($project)]
}

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::newProject" "Wiki::newProjectDetails"  --
 # 
 # Create a new project, using the info from the current page as the default
 # if available.  We then re-direct to 'Wiki::editProjects'.
 # 
 # "Wiki::editProjects" --
 # 
 # Open a dialog which contains pages for all projects, allowing the user to
 # create a new wiki project or to set home page urls and tag specifics. 
 # (Not all wikis use the same formatting rules.)  "Wiki::newProjectDetails"
 # is called by the dialog if the user is adding a new wiki project.
 # 
 # --------------------------------------------------------------------------
 ##

proc Wiki::newProject {} {
    
    set results  [Wiki::newProjectDetails]
    set name     [lindex $results 0]
    set settings [lindex $results 1]
    Wiki::projects $name $settings
    Wiki::currentProject $name
    Wiki::rebuildMenu
    Wiki::editProjects
}

proc Wiki::newProjectDetails {} {
    
    set p    "Descriptive name of new project"
    set name ""
    while {1} {
	if {[catch {prompt $p $name} name]} {
	    status::errorMsg "Cancelled."
	} elseif {![string length $name]} {
	    status::msg "No name was entered."
	} elseif {[lcontains projects $name]} {
	    alertnote "'$name' is already a defined Wiki project."
	    set p "Try another name"
	} else {
	    break
	}
    }
    if {[info exists ::Wiki::pages([win::Current])]} {
	set defaultHome [set ::Wiki::pages([win::Current])]
	regsub {/[0-9]+@} $defaultHome {/} defaultHome
	set theRest [lrange [Wiki::defaultTags] 4 end]]
	list $name [concat [list \
	  projectHome $defaultHome formatting ""] $theRest]
    } else {
	list $name [Wiki::defaultTags]
    }
}

proc Wiki::editProjects {} {

    Wiki::currentProject

    set results [dialog::editGroup -array Wiki::Projects -delete ask \
      -new Wiki::newProjectDetails \
      -title "Edit Wiki project information" \
      -current $project \
      [list "projectHome" url "Home Page"\
      "This is the base url containing the home page of the Wiki project."] \
      [list "formatting" url "Formatting Page"\
      "This is url that contains the project's formatting rules."] \
      [list "horizontalLine" variable "Horizontal Lines"\
      "This is the tag for horizontal lines."] \
      [list "bulletListItem" variable "Bulleted List Items"\
      "This is the tag for bulleted list items."] \
      [list "numberedItem" variable "Numbered Items"\
      "This is the tag for numbered items."] \
      [list "definitionItem" variable "Variable Items"\
      "This is the pair of tags for variable items."] \
      [list "verbatim" variable "Verbatim tags"\
      "This is the tag for verbatim text."] \
      [list "bold" variable "Bold tags"\
      "This is the pair of tags for bold text."] \
      [list "italics" variable "Italics tags"\
      "This is the pair of tags for italics text."] \
      ]
    if {![llength $results]} {return}
    # This works whether projects have been modified or deleted.
    foreach projectPage $results {
	if {![lcontains projects $projectPage]} {
	    # This is a new project, so we'll make it current.
	    Wiki::currentProject $projectPage
	}
	prefs::modified Wiki::Projects($projectPage)
    }
    # Make sure that the 'default' project wasn't deleted !!
    Wiki::projects "default" [Wiki::defaultTags]
    # Should now rebuild the menu in case we've added or removed a project.
    # 'Wiki::currentProject' will make sure that the current project still
    # exists, and adjust accordingly.
    Wiki::currentProject
    Wiki::rebuildMenu
}

proc Wiki::renameProject {} {
    
    global Wiki::Projects

    Wiki::currentProject

    set p "Rename which project?"
    while {1} {
	set projects [lremove $projects "default"]
	if {[catch {listpick -p $p -L [list $project] $projects} result]} {
	    status::errorMsg "Cancelled."
	} elseif {[catch {prompt "Rename '$result' to" "$result"} newName]} {
	    status::errorMsg "Cancelled."
	} elseif {[lcontains projects $newName]} {
	    alertnote "'$newName' is already defined as a project"
	} else {
	    set Wiki::Projects($newName) [set Wiki::Projects($result)]
	    unset Wiki::Projects($result)
	    prefs::modified Wiki::Projects($result)
	    Wiki::currentProject
	    if {![lcontains projects $project]} {
		Wiki::currentProject [lindex $projects 0]
	    } 
	    Wiki::rebuildMenu $project
	    set p "Choose another, or cancel"
	}
    }
}

proc Wiki::deleteProject {} {
    
    global Wiki::Projects
    
    set projects [lremove [Wiki::projects] "default"]
    if {[catch {listpick -p "Delete which projects?" -l $projects} results]} {
	status::errorMsg "Cancelled."
    } 
    foreach project $results {
	catch {unset Wiki::Projects($project)}
	prefs::modified Wiki::Projects($project)
    }
    Wiki::currentProject
    if {![lcontains projects $project]} {
	Wiki::currentProject [lindex $projects 0]
    } 
    Wiki::rebuildMenu $project
}

#  The Wiki Menu  #
# 
# Includes items for downloading wiki pages for editing, as well as text
# markup items.

menu::buildProc WikiMenu Wiki::buildWikiMenu Wiki::postMenuBuild

proc Wiki::buildWikiMenu {} {
    
    global WikiMenu global::features alpha::macos
    
    Wiki::currentProject
    
    set menuList [list "/t<BwikiModeHelp"]
    lappend menuList [list Menu -n wikiProjects -M Wiki -p "Wiki::menuProc" \
      [list "newProject" "editProjects" "renameProject" "deleteProject" ]]
    lappend menuList [menu::makeFlagMenu currentWiki \
      list project WikimodeVars {Wiki::rebuildMenu}]
    lappend menuList "(-)" "/w<U<B${project}Home" "/F<U<BformattingRules"
    if {[Wiki::urlsAllowed]} {
	lappend menuList "/E<U<BeditWikiPage" "saveToWeb" "(-)"
    } 
    # Mark-up tagging items.
    lappend menuList [list Menu -n "wikiParagraph" -M Wiki -p "Wiki::menuProc" \
      [list \
      /-<U<BhorizontalLine /*<U<BbulletListItem /1<U<BnumberedItem \
      /:<U<BdefinitionItem /V<E<S<U<Bverbatim   /V<S<O<U<Bunverbatim (-) \
      /P<U<BprevParagraph  /N<U<BnextParagraph  /S<U<BselectParagraph \
      /I<O<BreformatParagraph ]]
    if {[set alpha::macos]} {set b "<U<I"} else {set b "<U<B<O"}
    lappend menuList [list Menu -n "wikiLine" -M Wiki -p "Wiki::menuProc" \
      [list \
      /-${b}horizontalLine /*${b}bulletListItem /1${b}numberedItem ]]
    lappend menuList [list Menu -n "wikiText" -M Wiki -p "Wiki::menuProc" \
      [list \
      /B<U<Bbold /I<U<Bitalics /'<U<Bunquote ]]
    
    # Register any hooks if necessary.
    Wiki::registerHooks [lcontains global::features WikiMenu]
    
    # Now we finally create the menu.
    return [list build $menuList "Wiki::menuProc -M Wiki" "" $WikiMenu]
}

proc Wiki::postMenuBuild {args} {
    
    global WikiMenu
    
    Wiki::currentProject
    array set tagTypes $settings
    
    if {[llength $projects] == "1"} {
	enableMenuItem wikiProjects deleteProject 0
	enableMenuItem wikiProjects renameProject 0
    } 
    if {![string length $tagTypes(projectHome)]} {
	enableMenuItem $WikiMenu ${project}Home 0
    } 
    if {![string length $tagTypes(formatting)]} {
	enableMenuItem $WikiMenu formattingRules 0
    } 
}

proc Wiki::rebuildMenu {args} {

    menu::buildSome WikiMenu

    if {[llength $args]} {
	set arg [lindex $args 0]
	Wiki::currentProject
	if {$arg == "project" || [lcontains projects $arg]} {
	    status::msg "The current Wiki project is now '$project'"
	} else {
	    status::msg "The Wiki menu has been rebuilt."
	}
    }
}

proc Wiki::registerHooks {which} {
    
    global WikiMenu
    
    if {$which} {set which "register"} else {set which "deregister"}
    set items [list wikiParagraph wikiLine wikiText]
    if {[Wiki::urlsAllowed]} {lappend items saveToWeb}
    foreach item $items {
	hook::$which requireOpenWindowsHook [list $WikiMenu $item] 1
    }
}

# Now build the menu.
Wiki::rebuildMenu

#  Wiki Menu support  #

proc Wiki::menuProc {menuName itemName} {
    
    global WikiMenu
    
    Wiki::currentProject
    array set tagTypes $settings
    
    if {[regexp -nocase "${project}Home" $itemName]} {
	# Need to do this here because menu conversion might lowercase the
	# first character of the project name.
	if {![string length [set url $tagTypes(projectHome)]]} {
	    # Item should have been dimmed.
	    Wiki::postMenuBuild
	    status::errorMsg "Cancelled -- no home page defined."
	} else {
	    urlView $url
	    return
	}
    }
    regsub $WikiMenu $menuName "wikiMenu" menuName
    switch $menuName {
	"wikiMenu" - "wikiProjects" {
	    switch $itemName {
		"wikiModeHelp" {package::helpFile "Wiki"}
		"formattingRules" {
		    if {![string length [set url $tagTypes(formatting)]]} {
			# Item should have been dimmed.
			Wiki::postMenuBuild
			status::errorMsg "Cancelled -- no formatting page defined."
		    } else {
			urlView $url
		    }
		}
		"saveToWeb" {
		    # Note: a normal 'save' will always save to web if possible.
		    if {![info exists ::Wiki::page($name)]} {
			alertnote "This page was not fetched from a Wiki site, \
			  and cannot be saved to the web."
			status::errorMsg "Cancelled -- window cannot be saved to web."
		    }
		    if {![win::getInfo [win::Current] dirty]} {
			setWinInfo dirty 1 
		    } 
		    menu::fileProc "" "save"
		}
		default             {Wiki::${itemName}}
	    }
	}
	"wikiParagraph" {
	    switch $itemName {
		"horizontalLine" -
		"bulletListItem" -
		"numberedItem"      {Wiki::blockTag $menuName $itemName}
		"verbatim"       -  
		"unverbatim"        {Wiki::verbatimText $itemName}
		"prevParagraph"     {function::prev}
		"nextParagraph"     {function::next}
		"selectParagraph"   {function::select}
		default             {Wiki::${itemName}}
	    }
	}
	"wikiLine" {
	    switch $itemName {
		"horizontalLine" -
		"bulletListItem" -
		"numberedItem"      {Wiki::blockTag $menuName $itemName}
		default             {Wiki::${itemName}}
	    }
	}
	"wikiText" {
	    switch $itemName {
		"bold"           - 
		"italics"        -
		"unquote"           {Wiki::textTag $itemName}
		default             {Wiki::${itemName}}
	    }
	}
    }
}

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::editWikiPage" --
 # 
 # Offer a dialog to the user to fetch a remote wiki page.  The home page of
 # the current project is the default url presented, though the page in the
 # browser can also be used.  We take care of the url format here, which
 # allows us to determine the proper url for a rendered wiki page.
 # 
 # Only available in Tcl 8.0 and above.
 # 
 # --------------------------------------------------------------------------
 ##

proc Wiki::editWikiPage {{url ""}} {

    if {![Wiki::urlsAllowed]} {
	status::errorMsg "Sorry, editing remote urls is not available."
    } 
    currentProject

    if {![string length $url]} {
	set p  "Please type your url, or use one of the buttons below"
	if {![string length [set defaultUrl [lindex $settings 1]]]} {
	    set defaultUrl ""
	} 
	if {[catch {dialog::getUrl $p $defaultUrl} url]} {
	    status::errorMsg "Cancelled."
	} elseif {![string length $url]} {
	    status::errorMsg "Cancelled -- nothing was entered."
	}
    } 
    regsub -all {(@|\.htm(l)?|\*)$} $url {} url
    regsub {/$} $url {/0} url
    editUrl ${url}@
}

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::blockTag" --
 # 
 # Automatically insert a tag at the start of the block in which the cursor
 # currently resides.  Any previously highlighted selection is preserved. 
 # Tags can be inserted at the start of either the block or the current line.
 # 
 # --------------------------------------------------------------------------
 ##

proc Wiki::blockTag {menuName tagType} {
    
    Wiki::currentProject
    array set tagTypes $settings

    # Determine the tag to be inserted, with some special cases.
    if {[info exists tagTypes($tagType)]} {
	set tag [lindex $tagTypes($tagType) 0]
    } else {
	status::errorMsg "Unknown tag type: $tagType"
    }
    set posBeg [getPos]
    set posEnd [selEnd]
    set len1   [pos::diff $posBeg $posEnd]
    switch $menuName {
	"wikiParagraph" {set pos0 [Wiki::blockBeg $posBeg]}
	"wikiLine"      {set pos0 [pos::lineStart $posBeg]}
	default         {error "Unknown menu name: $menuName"}
    }
    set len2 [pos::diff $pos0 $posBeg]
    set pos1 [text::firstNonWsLinePos $pos0]
    set len3 [pos::diff $pos0 $pos1]
    # Replace the text, inserting the mark chars in front.  Can't simply
    # use 'replaceText' to insert the tag because Alpha7/8 won't properly
    # interpret a carriage return (glyph).
    replaceText $pos0 $pos1 ""
    goto $pos0
    insertText $tag
    goto [pos::math $pos0 + $len2 - $len3 + [string length $tag]]
    # Now re-hilite any original selection.
    select [getPos] [pos::math [getPos] + $len1]
    if {$menuName == "wikiParagraph"} {Wiki::menuProc "" "reformatParagraph"}
}

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::definitionItem" --
 # 
 # Delete any highlighted text, strip any whitespace surrounding the cursor,
 # insert the closing tag, and ensure that the beginning of the block
 # contains the open tag.
 # 
 # --------------------------------------------------------------------------
 ##

proc Wiki::definitionItem {} {
    
    Wiki::currentProject
    array set tagTypes $settings

    set openTag  [lindex $tagTypes(definitionItem) 0]
    set closeTag [lindex $tagTypes(definitionItem) 1]
    # If we're not in a paragraph block, life is easy.
    if {![lindex [function::inFunction [getPos]] 0]} {
	if {[isSelection]} {deleteSelection}
	replaceText [set pos [pos::lineStart]] [pos::lineEnd $pos] ""
	insertText ${openTag}${closeTag}
	return
    } 
    # Remove all of the spaces potentially between two words, and insert
    # the closing tag.
    set pos0 [getPos] ; set pos1 [selEnd]
    while {[regexp "\[\t \]" [getText $pos1 [pos::nextChar $pos1]]]} {
	set pos1 [pos::nextChar $pos1]
    }
    while {[regexp "\[\t \]" [getText [pos::prevChar $pos0] $pos0]]} {
	set pos0 [pos::prevChar $pos0]
    }
    replaceText $pos0 $pos1 $closeTag
    # Strip any leading indentation, and insert the open tag.
    set pos0 [Wiki::blockBeg [set pos [getPos]]]
    set pos1 [text::firstNonWsLinePos $pos0]
    set len1 [pos::diff $pos0 $pos]
    set len2 [pos::diff $pos0 $pos1]
    set len3 [string length $openTag]
    replaceText $pos0 $pos1 $openTag
    goto [pos::math $pos0 + $len1 - $len2 + $len3]
    status::msg "The 'definition' should be contained on one logical line."
}

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::verbatimText" --
 # 
 # Indent the entire block of text using the verbatim string, or remove all
 # spaces found in the first line of the block.  Retains any original
 # selection.
 # 
 # --------------------------------------------------------------------------
 ##

proc Wiki::verbatimText {tagType} {
    
    # Set the tag to be inserted/removed.
    switch -- $tagType {
	"verbatim" {
	    Wiki::currentProject
	    array set tagTypes $settings
	    set openTag [lindex $tagTypes($tagType) 0]
	} 
	"unverbatim" {
	    set pos0 [Wiki::blockBeg [getPos]]
	    set pos1 [text::firstNonWsLinePos $pos0]
	    set openTag [getText $pos0 $pos1]
	}
	default {error "Unknown tagtype: $tagType"}
    }
    # Preliminaries, remembering positions for later.
    set len1 [pos::diff [set pos0 [getPos]] [set pos1 [selEnd]]]
    set row0 [lindex [pos::toRowCol $pos0] 0]
    set row1 [lindex [pos::toRowCol $pos1] 0]
    set len2 [expr {($row1 - $row0) * [string length $openTag]}]
    if {![isSelection]} {
	function::select
    } else {
	select [Wiki::blockBeg [getPos]] [Wiki::blockEnd [selEnd]]
    }
    set memory [rememberWhereYouAre [set start [getPos]] $pos0 [selEnd]]
    # Change the prefix string, and (un)comment.
    global prefixString
    set oldPS $prefixString ; set prefixString $openTag
    switch $tagType {
	"verbatim"   {insertPrefix}
	"unverbatim" {removePrefix}
    }
    set prefixString $oldPS
    # Try to rehilite any previous selection.
    goBackToWhereYouWere $start \
      [pos::math $start + [string length [getText $start [getPos]]]] $memory
    switch $tagType {
	"verbatim"   {select [getPos] [pos::math [getPos] + $len1 + $len2]}
	"unverbatim" {select [getPos] [pos::math [getPos] + $len1 - $len2]}
    }
    # Finish up.
    status::msg "Text is now $tagType"
}

## 
 # --------------------------------------------------------------------------
 # 
 # "Wiki::textTag" --
 # 
 # Automatically insert a tag around the word surrounding the cursor, first
 # removing any similar tag characters surrounding the word if necessary. 
 # Retains any original selection.
 # 
 # --------------------------------------------------------------------------
 ##

proc Wiki::textTag {tagType} {
    
    Wiki::currentProject
    array set tagTypes $settings

    # Set the open/close tags.
    if {$tagType == "unquote"} {
	set openTag [set closeTag "'"]
    } elseif {[info exists tagTypes($tagType)]} {
	set openTag  [lindex $tagTypes($tagType) 0]
	set closeTag [lindex $tagTypes($tagType) 1]
    } else {
	status::errorMsg "Unknown tag type: $tagType"
    }
    # Preliminary position information.
    set pos  [getPos]
    set len1 [string length [getSelect]]
    eval select [text::surroundingWord]
    set word [getSelect]
    set len2 [pos::diff [set pos0 [getPos]] [set pos1 [selEnd]]]
    while {[regexp \\[lookAt [pos::prevChar $pos0]] $openTag]} {
	set pos0 [pos::prevChar $pos0]
    }
    while {[regexp \\[lookAt $pos1] $closeTag]} {
	set pos1 [pos::nextChar $pos1]
    }
    # Add the tag, and try to rehilite any previous selection.
    replaceText $pos0 $pos1 $word
    select $pos0 [pos::math $pos0 + [string length $word]]
    if {$tagType == "unquote"} {
	set openTag [set closeTag ""]
	set msg "'$word' has been unquoted."
    } elseif {[elec::Wrap ${openTag} ${closeTag}]} {
	set msg "'$word' has been set in $tagType."
    } else {
	set msg "Enter text to be set in $tagType."
    }
    set pos0 [pos::math $pos0 + $len2 + [string length $openTag]]
    set pos1 [pos::math $pos0 + $len1]
    select $pos0 $pos1
    status::msg $msg
}

#  Indentation, Filling, etc  #

## 
 # ----------------------------------------------------------------------
 #       
 # "Wiki::correctIndentation"  --
 #    
 # If we're at the start of a paragraph block, return the line's indentation. 
 # If we're in the second line of a block and the first line contains the
 # 'definitionItem' open and closing tags, return 0 in an attempt to conform
 # to 'logical line' rules.  (This might not work when the file is saved due
 # to the presence of hard carriage returns -- user probably should use the
 # menu item 'Paragraph To Line' before saving.)  Otherwise, return the
 # return the indentation of the most recent line, so that we can continue
 # indented tags if desired.
 # 
 # The main point here is to _not_ grab the indentation of the last line of
 # the previous paragraph if we're indenting or reformatting an entire
 # paragraph.
 # 
 # ----------------------------------------------------------------------
 ##

proc Wiki::correctIndentation {pos {nextChar ""}} {

    # If we're not in a paragraph block, return the indentation of the
    # previous line
    if {![lindex [set results [function::inFunction $pos]] 0]} {
	::correctIndentation $pos $nextChar
    } elseif {[pos::compare [pos::lineStart] == [set pos0 [lindex $results 1]]]} {
	# This is the first line of the block's indentation.
	set pos1 [text::firstNonWsLinePos $pos0]
	string length [text::maxSpaceForm [getText $pos0 $pos1]]
    } elseif {[pos::compare [pos::prevLineStart] == $pos0]} {
	# This is the second line, so check to see if we're in the middle of
	# a definition item.
	Wiki::currentProject
	array set tagTypes $settings
	set dIOpen  [lindex $tagTypes(definitionItem) 0]
	set dIClose [lindex $tagTypes(definitionItem) 1]
	set line    [getText $pos0 [pos::lineEnd $pos0]]
	if {[regexp ^$dIOpen $line] && [regexp $dIClose $line]} {
	    return 0
	} else {
	    ::correctIndentation $pos $nextChar
	}
    } else {
	::correctIndentation $pos $nextChar
    }
}

## 
 # ----------------------------------------------------------------------
 #       
 # "Wiki::paragraphFill" "Wiki::paragraphFillOne"  --
 #    
 # A variation of 'paragraph::fill' that takes our special definition of a
 # 'paragraph' into account.  Fill the current block, but first capture and
 # adjust the 'pad' in front.  Retains (well, approximately) any original
 # selection, selecting the lines that were filled.
 # 
 # --------------------------------------------------------------------------
 ##

proc Wiki::paragraphFill {} {

    if {![isSelection]} {
	Wiki::paragraphFillOne
    } else {	
	set pos0 [pos::lineStart [set pos [getPos]]]
	# If the selection extends to the start of the next line, we assume
	# that the end of the previous line was actually intended, otherwise
	# we need to select to the start of the next line.
	if {[pos::compare [set pos1 [selEnd]] != [pos::lineStart $pos1]]} {
	    set pos1 [pos::nextLineStart [selEnd]]
	} 
	while {[pos::compare $pos < $pos1]} {
	    goto $pos
	    set pos [Wiki::paragraphFillOne 0 $pos0 $pos1]
	    if {[pos::compare $pos >= [maxPos]]} {break}
	}
	select $pos0 [getPos]
    }
}

proc Wiki::paragraphFillOne {{remember 1} {minstart ""} {maxend ""}} {

    global leftFillColumn fillColumn doubleSpaces
    
    Wiki::currentProject
    set hl [lindex $settings 5]

    set pos  [getPos]
    set pos1 [pos::lineStart $pos]
    set pos2 [pos::lineEnd   $pos]
    # If we're on an empty or horizontal line, do nothing.
    set start [Wiki::blockBeg $pos]
    set end   [Wiki::blockEnd $start]
    if {![string length [string trim [set line [getText $pos1 $pos2]]]]} {
	return [pos::nextLineStart $pos2]
    } elseif {[regexp "^[string trim [join $hl]]\s*$" $line]} {
	return [pos::nextLineStart $pos2]
    }
    set start [Wiki::blockBeg $pos]
    set end   [Wiki::blockEnd $start]
    # Determine the pad in front.
    set pos3  [text::firstNonWsLinePos $start]
    set front [text::maxSpaceForm [getText $start $pos3]]
    regsub -all " +\t" $front "    " front
    set left  [string length $front]
    # Extra arguments allow us to specify a region in which to operate
    if {$minstart != "" && [pos::compare $minstart > $start]} {
	set start $minstart
    }
    if {$maxend != "" && [pos::compare $maxend < $end]} {
	set end $maxend
    }
    if {$remember} {
	set memory [rememberWhereYouAre $start $pos $end]
    }
    # Fill the text
    regsub -all "\[ \t\r\n\]+" [string trim [getText $start $end]] " " block
    # turn single spaces at end of sentences into double
    if {$doubleSpaces} {
	# Turn single spaces at end of sentences into double.
	set pat {(([^.][a-z]|[^a-zA-Z@]|\\@)[.?!]("|'|'')?([])])?) }
	regsub -all $pat $block {\1  } block
    }
    # Break and indent the paragraph
    regsub -all " ?\r" "\r[string trimright [breakIntoLines $block]]" "\r" block
    # Temporarily adjust the fillColumns, break and indent the block.
    set oldLeft $leftFillColumn ; set oldRight $fillColumn
    set leftFillColumn 0 ; set fillColumn [expr {$fillColumn - $left}]
    regsub -all " ?\r" $block "\r${front}" block
    # Reset columns
    set leftFillColumn $oldLeft ; set fillColumn $oldRight
    # Workaround an alpha bug
    if {$remember} { 
	set row [win::getInfo [win::Current] currline]
	if {[pos::compare [pos::fromRowCol $row 0] > $start]} {
	    goto $start
	}
    }
    replaceText $start $end "[string range $block 1 end]\r"
    if {$remember} {
	goBackToWhereYouWere \
	  $start [pos::math $start + [string length $block]] $memory
    }
    # In case we wish to fill a region
    pos::nextLineStart $end
}

## 
 # ----------------------------------------------------------------------
 #       
 # "Wiki::reformatParagraph"  --
 #    
 # A wrapper around 'function::reformat' that remembers where we are, and
 # makes sure that any horizontal lines touching the end of the paragraph
 # block are not inadvertantly indented.  (I think that's a bug in
 # 'function::reformat'.  In fact, the 'memory' stuff here probably should go
 # in there.)
 # 
 # ----------------------------------------------------------------------
 ##

proc Wiki::reformatParagraph {} {

    set len [pos::diff [set pos0 [getPos]] [set pos1 [selEnd]]]
    if {![isSelection]} {
	function::select
    } else {
	select [Wiki::blockBeg [getPos]] [Wiki::blockEnd [selEnd]]
    }
    set memory [rememberWhereYouAre [set start [getPos]] $pos0 [selEnd]]
    function::reformat
    # Make sure that we didn't indent a horizontal line.
    Wiki::currentProject
    array set tagTypes $settings
    set hl   [string trim [join $tagTypes(horizontalLine)]]
    set pos3 [pos::lineEnd [set pos2 [getPos]]]
    if {[regexp -- $hl [getText $pos2 $pos3]]} {
	replaceText [pos::lineStart $pos2] $pos3 ""
    } 
    # Try to select the previous selection.
    goBackToWhereYouWere $start \
      [pos::math $start + [string length [getText $start [getPos]]]] $memory
    select [getPos] [pos::math [getPos] + $len]
}

#  Wiki function:: support  #

## 
 # ----------------------------------------------------------------------
 #       
 # "Wiki::getLimits" --
 #    
 # Called by "function::" procs to determine the boundaries of any possible
 # paragraph blocks surrounding the cursor.  Allows us to simulate paragraph
 # navigation, selection, etc. while treating horizontal lines as 'empty',
 # indicating the separation of paragraphs.
 # 
 # ----------------------------------------------------------------------
 ##

proc Wiki::getLimits {pos direction} {

    set pos0 [Wiki::blockBeg $pos $direction]
    set pos1 [Wiki::blockEnd $pos0]
    
    list $pos0 $pos1 "text block"
}

##
 # ----------------------------------------------------------------------
 #       
 # "Wiki::blockBeg" --
 # "Wiki::blockEnd" --
 #      
 # Variations of paragraph::start/finish, used by Wiki::getLimits to navigate
 # and select blocks of text recognized as blocks of text that are separated
 # by empty lines, or by horizontal lines.
 # 
 # ----------------------------------------------------------------------
##

proc Wiki::blockBeg {pos {direction 0}} {
    
    if {!$direction && [pos::compare [pos::lineStart $pos] == [minPos]]} {
	return [minPos]
    } elseif {$direction && [pos::compare [pos::lineEnd $pos] == [maxPos]]} {
	return [maxPos]
    } elseif {$direction} {
	if {[pos::compare [pos::lineEnd $pos] == $pos]} {
	    set pos [pos::prevLineStart $pos]
	} else {
	    set pos [pos::lineStart $pos]
	}
    }
    
#   Wiki::currentProject
#   set    pat "(^[string trim [join [lindex $settings 5]]]\s*)|"
#   append pat {([\r\n]+[\r\n\t ]*[\r\n]+[^\r\n])}

    set pat {[-\r\n]+[-\r\n\t ]*[\r\n]+[^-\r\n]}
    if {![catch {search -f $direction -r 1 -s $pat $pos} match]} {
	set pos0 [pos::lineStart [lindex $match 1]]
    } elseif {!$direction} {
	set pos0 [minPos]
    } else {
	set pos0 [maxPos]
    }
    # We might have to back up one character and do this all over again if
    # we're greater than the original position.
    if {!$direction && [pos::compare $pos0 > $pos]} {
	Wiki::blockBeg [pos::prevChar $pos $direction]
    } elseif {$direction && [pos::compare $pos0 < $pos]} {
	Wiki::blockBeg [pos::nextLineStart $pos $direction]
    } else {
	set pos0
    }
}

proc Wiki::blockEnd {blockBeg} {
    
    set pos [pos::nextChar $blockBeg]

#   Wiki::currentProject
#   set    pat "(^[string trim [join [lindex $settings 5]]]\s*)|"
#   append pat {([\r\n]+[\r\n\t ]*[\r\n]+[^\r\n])}

    set pat {[-\r\n]+[-\r\n\t ]*[\r\n]+[^-\r\n]}
    set pos0 $pos
    set pos1 $pos
    # We might have to go one character forward and do this all over again if
    # the first iteration position is less than the original position.
    while {[pos::compare $pos0 <= $pos]} {
	if {[pos::compare [pos::lineEnd $pos1] == [maxPos]]} {
	    return [maxPos]
	} elseif {![catch {search -f 1 -r 1 -s $pat $pos1} match]} {
	    set pos0 [pos::nextLineStart [lindex $match 0]]
	} else {
	    return [maxPos]
	}
	set pos1 [pos::nextChar $pos1]
    }
    set pos0
}


