## -*-Tcl-*- (install)
 # ###################################################################
 # 
 #  FILE: "copyRing.tcl"
 #				    created: 3/11/94 {9:52:00 am} 
 #				last update: 02/09/2002 {11:26:07 AM} 
 #				
 # Implementation of Emacs's kill ring. This is a paste ring.
 # 
 # Package modified by Dominique d'Humieres
 # E-mail: <dominiq@physique.ens.fr>
 # following ideas from Juan Falgueras
 # E-mail: <juanfc@lcc.uma.es>
 # 
 # Various notes:
 # 
 # 1) To get the new 'smartCutPaste' behavior in TeX mode, the 'wordBreak'
 # must be set to '([\w:-]+|\\(\\|[a-zA-Z]+)\*?|\\[^a-zA-Z*\t\r\n\s])'.
 #
 # 2) If you prefer the old 'smartCutPaste' behavior, uncomment the last
 # commented paragraph at the end of the file. 
 # If you want to remove it, delete the code between '' and the lines:
 # 
 # ###################################################################
 ##

alpha::feature copyRing 0.2.1 global {
    # To insert/remove spaces to ensure a single space between words when
    # cutting or pasting, click this box||To cut/paste precisely what is 
    # selected, click this box.
    newPref flag smartCutPaste 1
    lappend flagPrefs(Text) smartCutPaste
} {
    menu::replaceWith Edit "/X<Scut" items "/X<Scut" "/X<S<I<Ocut&Append"
    menu::replaceWith Edit "/C<Scopy" items "/C<Scopy" "/C<S<I<Ocopy&Append"
    menu::replaceWith Edit "/V<Spaste" items "/V<Spaste" "/V<S<I<OpastePop"
    if {[info commands _preCopyringCopy] == ""} {
	set renamedRing 1
	rename copy _preCopyringCopy
	rename cut _preCopyringCut
	rename paste _preCopyringPaste
    }
    # to force loading of procs below
    auto_load copy&Append
    hook::register requireOpenWindowsHook [list Edit cut&Append] 1
    hook::register requireOpenWindowsHook [list Edit copy&Append] 1
    hook::register requireOpenWindowsHook [list Edit pastePop] 1
} {
    set renamedRing 0
    rename copy ""
    rename paste ""
    rename cut ""
    rename _preCopyringCopy copy
    rename _preCopyringCut cut
    rename _preCopyringPaste paste
    # so if we turn it on again we reload this file
    rename copy&Append ""
    hook::deregister requireOpenWindowsHook [list Edit cut&Append] 1
    hook::deregister requireOpenWindowsHook [list Edit copy&Append] 1
    hook::deregister requireOpenWindowsHook [list Edit pastePop] 1
} help {
    Provides an implementation of a copy/paste ring.
}

set ringDepth	5
set ringIn	0
set ringOut	0
set pasteStart	[minPos]
set pasteFinish	[minPos]

proc copy&Append {} {
    set old [getScrap]
    putScrap "$old[getSelect]"
    message "Appended"
}

proc cut&Append {} {
    global smartCutPaste

    set old [getScrap]
    set text [getSelect]
    putScrap "$old[getSelect]"
    deleteText [getPos] [selEnd]

#  If you don't want 'smartCutPaste' delete text from the previous bullet
#   to the next one
#
    if {$smartCutPaste} {
	if {[string trim "$text"] == "$text"} {
	    set gP [getPos]
	    if {[isNoWord $gP]} {
		if {[isWhite [pos::math $gP - 1]] && \
		    [isWordPrev  [pos::math $gP - 2]]} {
		    backSpace
		} else {
		    if {[isWhite $gP]} {
			deleteChar
		    }
		}
	    }
	}
    }
# 
    message "Appended"
}

proc cut {{rect 0}} {
    global copyring ringDepth ringIn smartCutPaste

    if {[pos::compare [getPos] == [selEnd]]} {
	if {[pos::compare [getMark] < [getPos]]} {
	    set text [getText [getMark] [getPos]]
	} else {
	    set text [getText [getPos] [getMark]]
	}
	if {![string length $text]} return
    } else {
	set text [getSelect]
    }

    set copyring([expr {$ringIn % $ringDepth}]) $text
    incr ringIn

    _preCopyringCut

#  If you don't want 'smartCutPaste' delete text from the previous bullet
#   to the next one
#
    if {$smartCutPaste && !$rect} {
	if {[string trim "$text"] == "$text"} {
	    set gP [getPos]
	    if {[isNoWord $gP]} {
		if {[isWhite [pos::math $gP - 1]] && \
		    [isWordPrev  [pos::math $gP - 2]]} {
		    backSpace
		} else {
		    if {[isWhite $gP]} {
			deleteChar
		    }
		}
	    }
	}
    }
# 
}

proc copy {} {
    global copyring ringDepth ringIn
    
    if {[pos::compare [getPos] == [selEnd]]} {
	if {[pos::compare [getMark] < [getPos]]} {
	    set text [getText [getMark] [getPos]]
	} else {
	    set text [getText [getPos] [getMark]]
	}
	if {![string length $text]} return
    } else {
	set text [getSelect]
    }
    
    set copyring([expr {$ringIn % $ringDepth}]) $text
    incr ringIn
    
    _preCopyringCopy
}

proc paste {{rect 0}} {
    global copyring ringDepth ringIn ringOut smartCutPaste pasteStart \
      pasteFinish
    set intel 0
    set ins 0
    set ringOut [expr {($ringIn - 1) % $ringDepth}]
    
#  If you don't want 'smartCutPaste' delete text from the previous bullet
#   to the next one
#
    if {!$rect && $smartCutPaste} {
	set scrap [getScrap]
	if {[isWhite [selEnd]] && [isWordPrev [pos::math [getPos] - 1]]\
	  && ([string trimleft $scrap] == $scrap)} {
	    clear
	    insertText " "
	    set ins 1
	} elseif {[isWhite [pos::math [getPos] -1]] && [isWordNext [selEnd]]\
	  && ([string trimright $scrap] == $scrap)} {
	    set intel 1
	}
    }
# 
    _preCopyringPaste
    set pasteStart [pos::math [getMark] - $ins]
    if {$intel} {insertText " "}
    set pasteFinish [getPos]
}
   
proc pastePop {} {
    global copyring ringDepth ringIn ringOut pasteFinish pasteStart \
      pasteScrap smartCutPaste
    
    if {!$ringIn} { beep; return}
    
    incr ringOut -1
    if {$ringOut < 0} {
	set ringOut [expr {(($ringDepth > $ringIn) ? $ringIn : $ringDepth) -1}]
    }
    
    set scrap $copyring($ringOut)
    
#  If you don't want 'smartCutPaste' delete text from the previous bullet
#   to the next one
#
    if {$smartCutPaste} {
	if {[isWhite $pasteFinish] && [isWordPrev [pos::math $pasteStart - 1]]\
	  && ([string trimleft $scrap] == $scrap)} {
	    set scrap " $scrap"
	} elseif {[isWhite [pos::math $pasteStart -1]] \
	  && [isWordNext $pasteFinish]\
	  && ([string trimright $scrap] == $scrap)} {
	    append scrap " "
	}
    }
# 
    set pasteScrap $scrap
    doCopyringPaste
    set pasteStart [getMark]
    set pasteFinish [getPos]
}

proc doCopyringPaste {} {
    global pasteFinish pasteStart pasteScrap ringOut

    #   added to remove the update of 'ringOut' by 'paste'
    set oldringOut $ringOut

    # pretend we are pasting just in case 'paste' has been
    # overridden (since that is the operation we are really
    # trying to imitate here).  This way we can operate
    # with the smartPaste package.
    set oldScrap [getScrap]
    putScrap $pasteScrap
    select $pasteStart $pasteFinish
    paste
    putScrap $oldScrap

    #   added to remove the update of 'ringOut' by 'paste'
    set ringOut $oldringOut
}

proc isWhite {p} {
    if {[pos::compare $p < [minPos]]} {
	set scrap [getScrap]
	if {([string trimright $scrap] == $scrap)} {
	    return 1
	} else {
	    return 0
	}
    } else {
	return [expr {([lookAt $p] == " ") || ([lookAt $p] == "\t")}]
    }
}

proc isChar {p} {
    return [expr {[string match {[a-z]} [lookAt $p]]}]
}

proc isWord {p} {
    global wordBreak
    return [regexp "$wordBreak" [lookAt $p]]
}

proc isNoWord {p} {
    global wordBreakPreface
    return [regexp "$wordBreakPreface" [lookAt $p]]
}

proc isWordNext {p} {
    global wordBreak
    if {[pos::compare $p < [maxPos]]} {
	if {[regexp "$wordBreak" [lookAt $p]]} {
	    return 1
	} elseif {[pos::compare [pos::math $p + 1] < [maxPos]]} {
	    set txt [getText $p [pos::math $p + 2]]	
	    if {[regexp "^$wordBreak" $txt]} {
		return 1
	    } elseif {[pos::compare [pos::math $p + 2] < [maxPos]]} {
		set txt [getText $p [pos::math $p + 3]]	
		return [regexp "^$wordBreak" $txt]
	    }
	}
    }
    return 0
}

proc isWordPrev {p} {
    global wordBreak
    global wordBreak
    if {[pos::compare $p > [minPos]]} {
	if {[regexp "$wordBreak" [lookAt $p]]} {
	    return 1
	} else {
	    set txt [getText [pos::math $p - 1] [pos::math $p + 1]]	
	    if {[regexp "^$wordBreak\$" $txt]} {
		return 1
	    } elseif {[pos::compare [pos::math $p - 1] > [minPos]]} {
		set txt [getText [pos::math $p - 2] [pos::math $p + 1]]	
		return [regexp "^$wordBreak\$" $txt]
	    }
	}
    }
    return 0
}

#
#   If you prefer the previous 'smartCutPaste' behavior uncomment the 
#   following paragraphs.
#

## 
 # proc isNoWord {p} {
 #     return [isWhite $p]
 # }
 # 
 # proc isWordNext {p} {
 #     return [isChar $p]
 # }
 # 
 # proc isWordPrev {p} {
 #     return [isChar $p]
 # }
 ##

